@@ -7,114 +7,21 @@
 //! [Wasmtime]: https://github.com/bytecodealliance/wasmtime
 
 use crate::state::FuncTranslationState;
-use crate::translation_utils::{
-    DataIndex, ElemIndex, EntityIndex, EntityType, Event, EventIndex, FuncIndex, Global,
-    GlobalIndex, InstanceIndex, InstanceTypeIndex, Memory, MemoryIndex, ModuleIndex,
-    ModuleTypeIndex, SignatureIndex, Table, TableIndex, TypeIndex,
+use crate::{
+    DataIndex, ElemIndex, EntityIndex, EntityType, FuncIndex, Global, GlobalIndex, InstanceIndex,
+    InstanceTypeIndex, Memory, MemoryIndex, ModuleIndex, ModuleTypeIndex, SignatureIndex, Table,
+    TableIndex, Tag, TagIndex, TypeIndex, WasmError, WasmFuncType, WasmResult, WasmType,
 };
 use core::convert::From;
-use core::convert::TryFrom;
 use cranelift_codegen::cursor::FuncCursor;
 use cranelift_codegen::ir::immediates::Offset32;
 use cranelift_codegen::ir::{self, InstBuilder};
 use cranelift_codegen::isa::TargetFrontendConfig;
 use cranelift_frontend::FunctionBuilder;
-#[cfg(feature = "enable-serde")]
-use serde::{Deserialize, Serialize};
 use std::boxed::Box;
 use std::string::ToString;
 use std::vec::Vec;
-use thiserror::Error;
-use wasmparser::ValidatorResources;
-use wasmparser::{BinaryReaderError, FuncValidator, FunctionBody, Operator, WasmFeatures};
-
-/// WebAssembly value type -- equivalent of `wasmparser`'s Type.
-#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
-#[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
-pub enum WasmType {
-    /// I32 type
-    I32,
-    /// I64 type
-    I64,
-    /// F32 type
-    F32,
-    /// F64 type
-    F64,
-    /// V128 type
-    V128,
-    /// FuncRef type
-    FuncRef,
-    /// ExternRef type
-    ExternRef,
-    /// ExnRef type
-    ExnRef,
-}
-
-impl TryFrom<wasmparser::Type> for WasmType {
-    type Error = WasmError;
-    fn try_from(ty: wasmparser::Type) -> Result<Self, Self::Error> {
-        use wasmparser::Type::*;
-        match ty {
-            I32 => Ok(WasmType::I32),
-            I64 => Ok(WasmType::I64),
-            F32 => Ok(WasmType::F32),
-            F64 => Ok(WasmType::F64),
-            V128 => Ok(WasmType::V128),
-            FuncRef => Ok(WasmType::FuncRef),
-            ExternRef => Ok(WasmType::ExternRef),
-            ExnRef => Ok(WasmType::ExnRef),
-            EmptyBlockType | Func => Err(WasmError::InvalidWebAssembly {
-                message: "unexpected value type".to_string(),
-                offset: 0,
-            }),
-        }
-    }
-}
-
-impl From<WasmType> for wasmparser::Type {
-    fn from(ty: WasmType) -> wasmparser::Type {
-        match ty {
-            WasmType::I32 => wasmparser::Type::I32,
-            WasmType::I64 => wasmparser::Type::I64,
-            WasmType::F32 => wasmparser::Type::F32,
-            WasmType::F64 => wasmparser::Type::F64,
-            WasmType::V128 => wasmparser::Type::V128,
-            WasmType::FuncRef => wasmparser::Type::FuncRef,
-            WasmType::ExternRef => wasmparser::Type::ExternRef,
-            WasmType::ExnRef => wasmparser::Type::ExnRef,
-        }
-    }
-}
-
-/// WebAssembly function type -- equivalent of `wasmparser`'s FuncType.
-#[derive(Debug, Clone, Eq, PartialEq, Hash)]
-#[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
-pub struct WasmFuncType {
-    /// Function params types.
-    pub params: Box<[WasmType]>,
-    /// Returns params types.
-    pub returns: Box<[WasmType]>,
-}
-
-impl TryFrom<wasmparser::FuncType> for WasmFuncType {
-    type Error = WasmError;
-    fn try_from(ty: wasmparser::FuncType) -> Result<Self, Self::Error> {
-        Ok(Self {
-            params: ty
-                .params
-                .into_vec()
-                .into_iter()
-                .map(WasmType::try_from)
-                .collect::<Result<_, Self::Error>>()?,
-            returns: ty
-                .returns
-                .into_vec()
-                .into_iter()
-                .map(WasmType::try_from)
-                .collect::<Result<_, Self::Error>>()?,
-        })
-    }
-}
+use wasmparser::{FuncValidator, FunctionBody, Operator, ValidatorResources, WasmFeatures};
 
 /// The value of a WebAssembly global variable.
 #[derive(Clone, Copy)]
@@ -136,64 +43,6 @@ pub enum GlobalVariable {
     Custom,
 }
 
-/// A WebAssembly translation error.
-///
-/// When a WebAssembly function can't be translated, one of these error codes will be returned
-/// to describe the failure.
-#[derive(Error, Debug)]
-pub enum WasmError {
-    /// The input WebAssembly code is invalid.
-    ///
-    /// This error code is used by a WebAssembly translator when it encounters invalid WebAssembly
-    /// code. This should never happen for validated WebAssembly code.
-    #[error("Invalid input WebAssembly code at offset {offset}: {message}")]
-    InvalidWebAssembly {
-        /// A string describing the validation error.
-        message: std::string::String,
-        /// The bytecode offset where the error occurred.
-        offset: usize,
-    },
-
-    /// A feature used by the WebAssembly code is not supported by the embedding environment.
-    ///
-    /// Embedding environments may have their own limitations and feature restrictions.
-    #[error("Unsupported feature: {0}")]
-    Unsupported(std::string::String),
-
-    /// An implementation limit was exceeded.
-    ///
-    /// Cranelift can compile very large and complicated functions, but the [implementation has
-    /// limits][limits] that cause compilation to fail when they are exceeded.
-    ///
-    /// [limits]: https://github.com/bytecodealliance/wasmtime/blob/main/cranelift/docs/ir.md#implementation-limits
-    #[error("Implementation limit exceeded")]
-    ImplLimitExceeded,
-
-    /// Any user-defined error.
-    #[error("User error: {0}")]
-    User(std::string::String),
-}
-
-/// Return an `Err(WasmError::Unsupported(msg))` where `msg` the string built by calling `format!`
-/// on the arguments to this macro.
-#[macro_export]
-macro_rules! wasm_unsupported {
-    ($($arg:tt)*) => { $crate::environ::WasmError::Unsupported(format!($($arg)*)) }
-}
-
-impl From<BinaryReaderError> for WasmError {
-    /// Convert from a `BinaryReaderError` to a `WasmError`.
-    fn from(e: BinaryReaderError) -> Self {
-        Self::InvalidWebAssembly {
-            message: e.message().into(),
-            offset: e.offset(),
-        }
-    }
-}
-
-/// A convenient alias for a `Result` that uses `WasmError` as the error type.
-pub type WasmResult<T> = Result<T, WasmError>;
-
 /// How to return from functions.
 #[derive(Copy, Clone, PartialEq, Eq, Debug)]
 pub enum ReturnMode {
@@ -613,6 +462,10 @@ pub trait FuncEnvironment: TargetEnvironment {
     /// for the same index.  Whether the waited-on value is 32- or 64-bit can be
     /// determined by examining the type of `expected`, which must be only I32 or I64.
     ///
+    /// Note that the `addr` here is the host linear memory address rather
+    /// than a relative wasm linear memory address. The type of this value is
+    /// the same as the host's pointer.
+    ///
     /// Returns an i32, which is negative if the helper call failed.
     fn translate_atomic_wait(
         &mut self,
@@ -629,6 +482,10 @@ pub trait FuncEnvironment: TargetEnvironment {
     /// to wait on, and `heap` is the heap reference returned by `make_heap`
     /// for the same index.
     ///
+    /// Note that the `addr` here is the host linear memory address rather
+    /// than a relative wasm linear memory address. The type of this value is
+    /// the same as the host's pointer.
+    ///
     /// Returns an i64, which is negative if the helper call failed.
     fn translate_atomic_notify(
         &mut self,
@@ -689,12 +546,16 @@ pub trait FuncEnvironment: TargetEnvironment {
     ) -> WasmResult<()> {
         Ok(())
     }
+
+    /// Returns the target ISA's condition to check for unsigned addition
+    /// overflowing.
+    fn unsigned_add_overflow_condition(&self) -> ir::condcodes::IntCC;
 }
 
 /// An object satisfying the `ModuleEnvironment` trait can be passed as argument to the
 /// [`translate_module`](fn.translate_module.html) function. These methods should not be called
 /// by the user, they are only for `cranelift-wasm` internal use.
-pub trait ModuleEnvironment<'data>: TargetEnvironment {
+pub trait ModuleEnvironment<'data> {
     /// Provides the number of types up front. By default this does nothing, but
     /// implementations can use this to preallocate memory if desired.
     fn reserve_types(&mut self, _num: u32) -> WasmResult<()> {
@@ -702,11 +563,7 @@ pub trait ModuleEnvironment<'data>: TargetEnvironment {
     }
 
     /// Declares a function signature to the environment.
-    fn declare_type_func(
-        &mut self,
-        wasm_func_type: WasmFuncType,
-        sig: ir::Signature,
-    ) -> WasmResult<()>;
+    fn declare_type_func(&mut self, wasm_func_type: WasmFuncType) -> WasmResult<()>;
 
     /// Declares a module type signature to the environment.
     fn declare_type_module(
@@ -775,15 +632,15 @@ pub trait ModuleEnvironment<'data>: TargetEnvironment {
         field: Option<&'data str>,
     ) -> WasmResult<()>;
 
-    /// Declares an event import to the environment.
-    fn declare_event_import(
+    /// Declares an tag import to the environment.
+    fn declare_tag_import(
         &mut self,
-        event: Event,
+        tag: Tag,
         module: &'data str,
         field: Option<&'data str>,
     ) -> WasmResult<()> {
-        drop((event, module, field));
-        Err(WasmError::Unsupported("wasm events".to_string()))
+        drop((tag, module, field));
+        Err(WasmError::Unsupported("wasm tags".to_string()))
     }
 
     /// Declares a global import to the environment.
@@ -848,16 +705,16 @@ pub trait ModuleEnvironment<'data>: TargetEnvironment {
     /// Declares a memory to the environment
     fn declare_memory(&mut self, memory: Memory) -> WasmResult<()>;
 
-    /// Provides the number of defined events up front. By default this does nothing, but
+    /// Provides the number of defined tags up front. By default this does nothing, but
     /// implementations can use this to preallocate memory if desired.
-    fn reserve_events(&mut self, _num: u32) -> WasmResult<()> {
+    fn reserve_tags(&mut self, _num: u32) -> WasmResult<()> {
         Ok(())
     }
 
-    /// Declares an event to the environment
-    fn declare_event(&mut self, event: Event) -> WasmResult<()> {
-        drop(event);
-        Err(WasmError::Unsupported("wasm events".to_string()))
+    /// Declares an tag to the environment
+    fn declare_tag(&mut self, tag: Tag) -> WasmResult<()> {
+        drop(tag);
+        Err(WasmError::Unsupported("wasm tags".to_string()))
     }
 
     /// Provides the number of defined globals up front. By default this does nothing, but
@@ -889,14 +746,10 @@ pub trait ModuleEnvironment<'data>: TargetEnvironment {
         name: &'data str,
     ) -> WasmResult<()>;
 
-    /// Declares an event export to the environment.
-    fn declare_event_export(
-        &mut self,
-        event_index: EventIndex,
-        name: &'data str,
-    ) -> WasmResult<()> {
-        drop((event_index, name));
-        Err(WasmError::Unsupported("wasm events".to_string()))
+    /// Declares an tag export to the environment.
+    fn declare_tag_export(&mut self, tag_index: TagIndex, name: &'data str) -> WasmResult<()> {
+        drop((tag_index, name));
+        Err(WasmError::Unsupported("wasm tags".to_string()))
     }
 
     /// Declares a global export to the environment.
@@ -941,7 +794,7 @@ pub trait ModuleEnvironment<'data>: TargetEnvironment {
         &mut self,
         table_index: TableIndex,
         base: Option<GlobalIndex>,
-        offset: usize,
+        offset: u32,
         elements: Box<[FuncIndex]>,
     ) -> WasmResult<()>;
 
@@ -952,6 +805,13 @@ pub trait ModuleEnvironment<'data>: TargetEnvironment {
         elements: Box<[FuncIndex]>,
     ) -> WasmResult<()>;
 
+    /// Indicates that a declarative element segment was seen in the wasm
+    /// module.
+    fn declare_elements(&mut self, elements: Box<[FuncIndex]>) -> WasmResult<()> {
+        drop(elements);
+        Ok(())
+    }
+
     /// Provides the number of passive data segments up front.
     ///
     /// By default this does nothing, but implementations may use this to
@@ -988,7 +848,7 @@ pub trait ModuleEnvironment<'data>: TargetEnvironment {
         &mut self,
         memory_index: MemoryIndex,
         base: Option<GlobalIndex>,
-        offset: usize,
+        offset: u64,
         data: &'data [u8],
     ) -> WasmResult<()>;
 
